def list_known_markup_functions():
    '''List all markup functions recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_markup_functions():
        ...     print x
        ...
        abs-fontsize
        arrow-head
        auto-footnote
        backslashed-digit
        beam
        bold
        box
        bracket
        caps
        center-align
        center-column
        char
        circle
        column
        column-lines
        combine
        concat
        customTabClef
        dir-column
        doubleflat
        doublesharp
        draw-circle
        draw-hline
        draw-line
        dynamic
        epsfile
        eyeglasses
        fill-line
        fill-with-pattern
        filled-box
        finger
        flat
        fontCaps
        fontsize
        footnote
        fraction
        fret-diagram
        fret-diagram-terse
        fret-diagram-verbose
        fromproperty
        general-align
        halign
        harp-pedal
        hbracket
        hcenter-in
        hspace
        huge
        italic
        justified-lines
        justify
        justify-field
        justify-string
        large
        larger
        left-align
        left-brace
        left-column
        line
        lookup
        lower
        magnify
        markalphabet
        markletter
        medium
        musicglyph
        natural
        normal-size-sub
        normal-size-super
        normal-text
        normalsize
        note
        note-by-number
        null
        number
        on-the-fly
        override
        override-lines
        pad
        pad-around
        pad-to-box
        pad-x
        page-link
        page-ref
        parenthesize
        path
        pattern
        postscript
        property-recursive
        put-adjacent
        raise
        replace
        rest
        rest-by-number
        right-align
        right-brace
        right-column
        roman
        rotate
        rounded-box
        sans
        scale
        score
        semiflat
        semisharp
        sesquiflat
        sesquisharp
        sharp
        simple
        slashed-digit
        small
        smallCaps
        smaller
        stencil
        strut
        sub
        super
        table-of-contents
        teeny
        text
        tied-lyric
        tiny
        translate
        translate-scaled
        transparent
        triangle
        typewriter
        underline
        upright
        vcenter
        verbatim-file
        vspace
        whiteout
        with-color
        with-dimensions
        with-link
        with-url
        woodwind-diagram
        wordwrap
        wordwrap-field
        wordwrap-internal
        wordwrap-lines
        wordwrap-string
        wordwrap-string-internal

    Return list.
    '''
    from abjad.ly import markup_functions
    from abjad.ly import markup_list_functions

    return sorted(markup_functions.keys() + markup_list_functions.keys())
