def list_known_grobs():
    '''List all LilyPond grobs recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_grobs():
        ...     print x
        ...
        Accidental
        AccidentalCautionary
        AccidentalPlacement
        AccidentalSuggestion
        Ambitus
        AmbitusAccidental
        AmbitusLine
        AmbitusNoteHead
        Arpeggio
        BalloonTextItem
        BarLine
        BarNumber
        BassFigure
        BassFigureAlignment
        BassFigureAlignmentPositioning
        BassFigureBracket
        BassFigureContinuation
        BassFigureLine
        Beam
        BendAfter
        BreakAlignGroup
        BreakAlignment
        BreathingSign
        ChordName
        Clef
        ClusterSpanner
        ClusterSpannerBeacon
        CombineTextScript
        CueClef
        CueEndClef
        Custos
        DotColumn
        Dots
        DoublePercentRepeat
        DoublePercentRepeatCounter
        DoubleRepeatSlash
        DynamicLineSpanner
        DynamicText
        DynamicTextSpanner
        Episema
        Fingering
        FingeringColumn
        Flag
        FootnoteItem
        FootnoteSpanner
        FretBoard
        Glissando
        GraceSpacing
        GridLine
        GridPoint
        Hairpin
        HorizontalBracket
        InstrumentName
        InstrumentSwitch
        KeyCancellation
        KeySignature
        LaissezVibrerTie
        LaissezVibrerTieColumn
        LedgerLineSpanner
        LeftEdge
        LigatureBracket
        LyricExtender
        LyricHyphen
        LyricSpace
        LyricText
        MeasureCounter
        MeasureGrouping
        MelodyItem
        MensuralLigature
        MetronomeMark
        MultiMeasureRest
        MultiMeasureRestNumber
        MultiMeasureRestText
        NonMusicalPaperColumn
        NoteCollision
        NoteColumn
        NoteHead
        NoteName
        NoteSpacing
        OctavateEight
        OttavaBracket
        PaperColumn
        ParenthesesItem
        PercentRepeat
        PercentRepeatCounter
        PhrasingSlur
        PianoPedalBracket
        RehearsalMark
        RepeatSlash
        RepeatTie
        RepeatTieColumn
        Rest
        RestCollision
        Script
        ScriptColumn
        ScriptRow
        Slur
        SostenutoPedal
        SostenutoPedalLineSpanner
        SpacingSpanner
        SpanBar
        SpanBarStub
        StaffGrouper
        StaffSpacing
        StaffSymbol
        StanzaNumber
        Stem
        StemStub
        StemTremolo
        StringNumber
        StrokeFinger
        SustainPedal
        SustainPedalLineSpanner
        System
        SystemStartBar
        SystemStartBrace
        SystemStartBracket
        SystemStartSquare
        TabNoteHead
        TextScript
        TextSpanner
        Tie
        TieColumn
        TimeSignature
        TrillPitchAccidental
        TrillPitchGroup
        TrillPitchHead
        TrillSpanner
        TupletBracket
        TupletNumber
        UnaCordaPedal
        UnaCordaPedalLineSpanner
        VaticanaLigature
        VerticalAlignment
        VerticalAxisGroup
        VoiceFollower
        VoltaBracket
        VoltaBracketSpanner

    Return tuple.
    '''
    from abjad.ly import grob_interfaces

    return sorted(grob_interfaces.keys())
